@@ -1803,9 +1803,11 @@ _get_group(preference_t pref)
     case PREF_WINTITLE_GOODBYE:
     case PREF_FLASH:
     case PREF_INTYPE:
+    case PREF_INTYPE_CONSOLE:
     case PREF_HISTORY:
     case PREF_OCCUPANTS:
     case PREF_OCCUPANTS_JID:
+    case PREF_OCCUPANTS_OFFLINE:
     case PREF_OCCUPANTS_WRAP:
     case PREF_STATUSES:
     case PREF_STATUSES_CONSOLE:
@@ -1867,6 +1869,7 @@ _get_group(preference_t pref)
     case PREF_TITLEBAR_MUC_TITLE_JID:
     case PREF_TITLEBAR_MUC_TITLE_NAME:
     case PREF_SLASH_GUARD:
+    case PREF_COMPOSE_EDITOR:
         return PREF_GROUP_UI;
     case PREF_STATES:
     case PREF_OUTTYPE:
@@ -1913,6 +1916,7 @@ _get_group(preference_t pref)
     case PREF_TLS_CERTPATH:
     case PREF_CORRECTION_ALLOW:
     case PREF_MAM:
+    case PREF_SILENCE_NON_ROSTER:
         return PREF_GROUP_CONNECTION;
     case PREF_OTR_LOG:
     case PREF_OTR_POLICY:
@@ -1928,6 +1932,7 @@ _get_group(preference_t pref)
         return PREF_GROUP_PLUGINS;
     case PREF_OMEMO_LOG:
     case PREF_OMEMO_POLICY:
+    case PREF_OMEMO_TRUST_MODE:
         return PREF_GROUP_OMEMO;
     default:
         return NULL;
@@ -1964,6 +1969,8 @@ _get_key(preference_t pref)
         return "adv.notify.discoversion";
     case PREF_INTYPE:
         return "intype";
+    case PREF_INTYPE_CONSOLE:
+        return "intype.console";
     case PREF_HISTORY:
         return "history";
     case PREF_CARBONS:
@@ -1978,6 +1985,8 @@ _get_key(preference_t pref)
         return "occupants";
     case PREF_OCCUPANTS_JID:
         return "occupants.jid";
+    case PREF_OCCUPANTS_OFFLINE:
+        return "occupants.offline";
     case PREF_OCCUPANTS_WRAP:
         return "occupants.wrap";
     case PREF_MUC_PRIVILEGES:
@@ -2174,6 +2183,8 @@ _get_key(preference_t pref)
         return "log";
     case PREF_OMEMO_POLICY:
         return "policy";
+    case PREF_OMEMO_TRUST_MODE:
+        return "trustmode";
     case PREF_CORRECTION_ALLOW:
         return "correction.allow";
     case PREF_AVATAR_CMD:
@@ -2186,6 +2197,10 @@ _get_key(preference_t pref)
         return "url.open.cmd";
     case PREF_URL_SAVE_CMD:
         return "url.save.cmd";
+    case PREF_COMPOSE_EDITOR:
+        return "compose.editor";
+    case PREF_SILENCE_NON_ROSTER:
+        return "silence.incoming.nonroster";
     default:
         return NULL;
     }
@@ -2201,14 +2216,6 @@ _get_default_boolean(preference_t pref)
     case PREF_AUTOAWAY_CHECK:
     case PREF_LOG_ROTATE:
     case PREF_LOG_SHARED:
-    case PREF_NOTIFY_CHAT:
-    case PREF_NOTIFY_CHAT_CURRENT:
-    case PREF_NOTIFY_ROOM:
-    case PREF_NOTIFY_ROOM_CURRENT:
-    case PREF_NOTIFY_TYPING:
-    case PREF_NOTIFY_TYPING_CURRENT:
-    case PREF_NOTIFY_SUB:
-    case PREF_NOTIFY_INVITE:
     case PREF_SPLASH:
     case PREF_OCCUPANTS:
     case PREF_MUC_PRIVILEGES:
@@ -2229,7 +2236,6 @@ _get_default_boolean(preference_t pref)
     case PREF_ROSTER_ROOMS_SERVER:
     case PREF_TLS_SHOW:
     case PREF_LASTACTIVITY:
-    case PREF_NOTIFY_MENTION_WHOLE_WORD:
     case PREF_TRAY_READ:
     case PREF_BOOKMARK_INVITE:
     case PREF_ROOM_LIST_CACHE:
@@ -2243,6 +2249,8 @@ _get_default_boolean(preference_t pref)
     case PREF_OUTTYPE:
     case PREF_TITLEBAR_MUC_TITLE_NAME:
     case PREF_COLOR_NICK_OWN:
+    case PREF_INTYPE:
+    case PREF_INTYPE_CONSOLE:
         return TRUE;
     default:
         return FALSE;
@@ -2317,12 +2325,16 @@ _get_default_string(preference_t pref)
         return "redact";
     case PREF_OMEMO_POLICY:
         return "automatic";
+    case PREF_OMEMO_TRUST_MODE:
+        return "manual";
     case PREF_COLOR_NICK:
         return "false";
     case PREF_AVATAR_CMD:
         return "xdg-open";
     case PREF_URL_OPEN_CMD:
         return "xdg-open %u";
+    case PREF_COMPOSE_EDITOR:
+        return "vim";
     case PREF_URL_SAVE_CMD:
         return NULL; // Default to built-in method.
     default:
