@@ -33,6 +33,8 @@
  * source files in the program, then also delete it here.
  *
  */
+#include "config.h"
+
 #include <sys/time.h>
 #include <sys/stat.h>
 
@@ -118,7 +120,7 @@ static omemo_context omemo_ctx;
 void
 omemo_init(void)
 {
-    log_info("OMEMO: initialising");
+    log_info("[OMEMO] initialising");
     if (omemo_crypto_init() != 0) {
         cons_show("Error initializing OMEMO crypto: gcry_check_version() failed");
     }
@@ -244,9 +246,9 @@ omemo_on_connect(ProfAccount* account)
     if (res == -1) {
         const char* errmsg = strerror(errno);
         if (errmsg) {
-            log_error("OMEMO: error creating directory: %s, %s", omemo_dir, errmsg);
+            log_error("[OMEMO] error creating directory: %s, %s", omemo_dir, errmsg);
         } else {
-            log_error("OMEMO: creating directory: %s", omemo_dir);
+            log_error("[OMEMO] creating directory: %s", omemo_dir);
         }
     }
 
@@ -264,7 +266,7 @@ omemo_on_connect(ProfAccount* account)
             return;
         }
     } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("OMEMO: error loading identity from: %s, %s", omemo_ctx.identity_filename->str, error->message);
+        log_warning("[OMEMO] error loading identity from: %s, %s", omemo_ctx.identity_filename->str, error->message);
         g_error_free(error);
         return;
     }
@@ -273,7 +275,7 @@ omemo_on_connect(ProfAccount* account)
     if (g_key_file_load_from_file(omemo_ctx.trust_keyfile, omemo_ctx.trust_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
         _load_trust();
     } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("OMEMO: error loading trust from: %s, %s", omemo_ctx.trust_filename->str, error->message);
+        log_warning("[OMEMO] error loading trust from: %s, %s", omemo_ctx.trust_filename->str, error->message);
         g_error_free(error);
     }
 
@@ -281,7 +283,7 @@ omemo_on_connect(ProfAccount* account)
     if (g_key_file_load_from_file(omemo_ctx.sessions_keyfile, omemo_ctx.sessions_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
         _load_sessions();
     } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("OMEMO: error loading sessions from: %s, %s", omemo_ctx.sessions_filename->str, error->message);
+        log_warning("[OMEMO] error loading sessions from: %s, %s", omemo_ctx.sessions_filename->str, error->message);
         g_error_free(error);
     }
 
@@ -289,7 +291,7 @@ omemo_on_connect(ProfAccount* account)
     if (g_key_file_load_from_file(omemo_ctx.known_devices_keyfile, omemo_ctx.known_devices_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
         _load_known_devices();
     } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("OMEMO: error loading known devices from: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
+        log_warning("[OMEMO] error loading known devices from: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
         g_error_free(error);
     }
 }
@@ -329,7 +331,7 @@ omemo_generate_crypto_materials(ProfAccount* account)
     gcry_randomize(&omemo_ctx.device_id, 4, GCRY_VERY_STRONG_RANDOM);
     omemo_ctx.device_id &= 0x7fffffff;
     g_key_file_set_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, omemo_ctx.device_id);
-    log_info("OMEMO: device id: %d", omemo_ctx.device_id);
+    log_info("[OMEMO] device id: %d", omemo_ctx.device_id);
 
     /* Identity key */
     signal_protocol_key_helper_generate_identity_key_pair(&omemo_ctx.identity_key_pair, omemo_ctx.signal);
@@ -365,8 +367,11 @@ omemo_generate_crypto_materials(ProfAccount* account)
 void
 omemo_publish_crypto_materials(void)
 {
+    log_debug("[OMEMO] publish crypto materials");
+
     if (loaded != TRUE) {
-        log_error("OMEMO: cannot publish crypto materials before they are generated");
+        cons_show("OMEMO: cannot publish crypto materials before they are generated");
+        log_error("[OMEMO] cannot publish crypto materials before they are generated");
         return;
     }
 
@@ -401,12 +406,18 @@ void
 omemo_start_session(const char* const barejid)
 {
     if (omemo_loaded()) {
-        log_info("OMEMO: start session with %s", barejid);
+        log_debug("[OMEMO] start session with %s", barejid);
         GList* device_list = g_hash_table_lookup(omemo_ctx.device_list, barejid);
         if (!device_list) {
-            log_info("OMEMO: missing device list for %s", barejid);
+            log_debug("[OMEMO] missing device list for %s", barejid);
+            // Own devices are handled by _handle_own_device_list
+            // We won't add _handle_device_list_start_session for ourself
+            char* mybarejid = connection_get_barejid();
+            if( g_strcmp0(mybarejid, barejid ) != 0 ) {
+                g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_device_list_start_session);
+            }
+            free(mybarejid);
             omemo_devicelist_request(barejid);
-            g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_device_list_start_session);
             return;
         }
 
@@ -522,6 +533,7 @@ omemo_prekeys(GList** prekeys, GList** ids, GList** lengths)
 void
 omemo_set_device_list(const char* const from, GList* device_list)
 {
+    log_debug("[OMEMO] Setting device list for %s", from);
     Jid* jid;
     if (from) {
         jid = jid_create(from);
@@ -537,8 +549,38 @@ omemo_set_device_list(const char* const from, GList* device_list)
         if (!keep) {
             g_hash_table_remove(omemo_ctx.device_list_handler, jid->barejid);
         }
+    } else {
+        log_debug("[OMEMO] No Device List Handler for %s", from);
     }
 
+    // OMEMO trustmode ToFu
+    if (g_strcmp0(prefs_get_string(PREF_OMEMO_TRUST_MODE), "firstusage") == 0) {
+        log_debug("[OMEMO] Checking firstusage state for %s", jid->barejid);
+        GHashTable* trusted = g_hash_table_lookup(omemo_ctx.identity_key_store.trusted, jid->barejid);
+        if (trusted) {
+            if (g_hash_table_size(trusted) > 0) {
+                log_debug("[OMEMO] Found trusted device for %s - skip firstusage", jid->barejid);
+                return;
+            }
+        } else {
+            if (device_list) {
+                cons_show("OMEMO: No trusted devices found for %s", jid->barejid);
+                GList* device_id;
+                for (device_id = device_list; device_id != NULL; device_id = device_id->next) {
+                    GHashTable* known_identities = g_hash_table_lookup(omemo_ctx.known_devices, jid->barejid);
+                    if (known_identities) {
+                        GList* fp = NULL;
+                        for (fp = g_hash_table_get_keys(known_identities); fp != NULL; fp = fp->next) {
+                            if (device_id->data == g_hash_table_lookup(known_identities, fp->data)) {
+                                cons_show("OMEMO: Adding firstusage trust for %s device %d - Fingerprint %s", jid->barejid, device_id->data, omemo_format_fingerprint(fp->data));
+                                omemo_trust(jid->barejid, omemo_format_fingerprint(fp->data));
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
     jid_destroy(jid);
 }
 
@@ -554,7 +596,7 @@ omemo_identity_keyfile_save(void)
     GError* error = NULL;
 
     if (!g_key_file_save_to_file(omemo_ctx.identity_keyfile, omemo_ctx.identity_filename->str, &error)) {
-        log_error("OMEMO: error saving identity to: %s, %s", omemo_ctx.identity_filename->str, error->message);
+        log_error("[OMEMO] error saving identity to: %s, %s", omemo_ctx.identity_filename->str, error->message);
     }
 }
 
@@ -570,7 +612,7 @@ omemo_trust_keyfile_save(void)
     GError* error = NULL;
 
     if (!g_key_file_save_to_file(omemo_ctx.trust_keyfile, omemo_ctx.trust_filename->str, &error)) {
-        log_error("OMEMO: error saving trust to: %s, %s", omemo_ctx.trust_filename->str, error->message);
+        log_error("[OMEMO] error saving trust to: %s, %s", omemo_ctx.trust_filename->str, error->message);
     }
 }
 
@@ -586,7 +628,7 @@ omemo_sessions_keyfile_save(void)
     GError* error = NULL;
 
     if (!g_key_file_save_to_file(omemo_ctx.sessions_keyfile, omemo_ctx.sessions_filename->str, &error)) {
-        log_error("OMEMO: error saving sessions to: %s, %s", omemo_ctx.sessions_filename->str, error->message);
+        log_error("[OMEMO] error saving sessions to: %s, %s", omemo_ctx.sessions_filename->str, error->message);
     }
 }
 
@@ -596,7 +638,7 @@ omemo_known_devices_keyfile_save(void)
     GError* error = NULL;
 
     if (!g_key_file_save_to_file(omemo_ctx.known_devices_keyfile, omemo_ctx.known_devices_filename->str, &error)) {
-        log_error("OMEMO: error saving known devices to: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
+        log_error("[OMEMO] error saving known devices to: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
     }
 }
 
@@ -607,6 +649,7 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
                            const unsigned char* const signature, size_t signature_len,
                            const unsigned char* const identity_key_raw, size_t identity_key_len)
 {
+    log_debug("[OMEMO] Starting device session for %s with device %d", jid, device_id);
     signal_protocol_address address = {
         .name = jid,
         .name_len = strlen(jid),
@@ -618,12 +661,23 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
     _cache_device_identity(jid, device_id, identity_key);
 
     gboolean trusted = is_trusted_identity(&address, (uint8_t*)identity_key_raw, identity_key_len, &omemo_ctx.identity_key_store);
+    log_debug("[OMEMO] Trust %s (%d): %d", jid, device_id, trusted);
+
+    if ((g_strcmp0(prefs_get_string(PREF_OMEMO_TRUST_MODE), "blind") == 0) && !trusted) {
+        char* fp = _omemo_fingerprint(identity_key, TRUE);
+        cons_show("Blind trust for %s device %d (%s)", jid, device_id, fp);
+        omemo_trust(jid, fp);
+        free(fp);
+        trusted = TRUE;
+    }
 
     if (!trusted) {
+        log_debug("[OMEMO] We don't trust device %d for %s\n", device_id,jid);
         goto out;
     }
 
     if (!contains_session(&address, omemo_ctx.session_store)) {
+        log_debug("[OMEMO] There is no Session for %s ( %d) ,... building session.", address.name, address.device_id );
         int res;
         session_pre_key_bundle* bundle;
         signal_protocol_address* address;
@@ -636,7 +690,7 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
         session_builder* builder;
         res = session_builder_create(&builder, omemo_ctx.store, address, omemo_ctx.signal);
         if (res != 0) {
-            log_error("OMEMO: cannot create session builder for %s device %d", jid, device_id);
+            log_error("[OMEMO] cannot create session builder for %s device %d", jid, device_id);
             goto out;
         }
 
@@ -652,17 +706,19 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
 
         res = session_pre_key_bundle_create(&bundle, 0, device_id, prekey->id, prekey_public, signed_prekey_id, signed_prekey, signature, signature_len, identity_key);
         if (res != 0) {
-            log_error("OMEMO: cannot create pre key bundle for %s device %d", jid, device_id);
+            log_error("[OMEMO] cannot create pre key bundle for %s device %d", jid, device_id);
             goto out;
         }
 
         res = session_builder_process_pre_key_bundle(builder, bundle);
         if (res != 0) {
-            log_error("OMEMO: cannot process pre key bundle for %s device %d", jid, device_id);
+            log_error("[OMEMO] cannot process pre key bundle for %s device %d", jid, device_id);
             goto out;
         }
 
-        log_info("OMEMO: create session with %s device %d", jid, device_id);
+        log_debug("[OMEMO] create session with %s device %d", jid, device_id);
+    } else {
+        log_debug("[OMEMO] session with %s device %d exists", jid, device_id);
     }
 
 out:
@@ -695,13 +751,14 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
 
     res = aes128gcm_encrypt(ciphertext, &ciphertext_len, tag, &tag_len, (const unsigned char* const)message, strlen(message), iv, key);
     if (res != 0) {
-        log_error("OMEMO: cannot encrypt message");
+        log_error("[OMEMO][SEND] cannot encrypt message");
         goto out;
     }
 
     memcpy(key_tag, key, AES128_GCM_KEY_LENGTH);
     memcpy(key_tag + AES128_GCM_KEY_LENGTH, tag, AES128_GCM_TAG_LENGTH);
 
+    // List of barejids of the recipients of this message
     GList* recipients = NULL;
     if (muc) {
         ProfMucWin* mucwin = (ProfMucWin*)win;
@@ -724,12 +781,14 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
 
     omemo_ctx.identity_key_store.recv = false;
 
+    // Encrypt keys for the recipients
     GList* recipients_iter;
     for (recipients_iter = recipients; recipients_iter != NULL; recipients_iter = recipients_iter->next) {
         GList* recipient_device_id = NULL;
         recipient_device_id = g_hash_table_lookup(omemo_ctx.device_list, recipients_iter->data);
         if (!recipient_device_id) {
-            log_warning("OMEMO: cannot find device ids for %s", recipients_iter->data);
+            log_warning("[OMEMO][SEND] cannot find device ids for %s", recipients_iter->data);
+            win_println(win, THEME_ERROR, "!", "Can't find a OMEMO device id for %s.\n", recipients_iter->data);
             continue;
         }
 
@@ -743,16 +802,30 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
                 .device_id = GPOINTER_TO_INT(device_ids_iter->data)
             };
 
+            // Don't encrypt for this device (according to
+            // <https://xmpp.org/extensions/xep-0384.html#encrypt>).
+            // Yourself as recipients in case of MUC
+            char* mybarejid = connection_get_barejid();
+            if ( !g_strcmp0(mybarejid, recipients_iter->data) ) {
+                if (GPOINTER_TO_INT(device_ids_iter->data) == omemo_ctx.device_id) {
+                    free(mybarejid);
+                    log_debug("[OMEMO][SEND] Skipping %d (my device) ", GPOINTER_TO_INT(device_ids_iter->data));
+                    continue;
+                }
+            }
+            free(mybarejid);
+
+            log_debug("[OMEMO][SEND] recipients with device id %d for %s", GPOINTER_TO_INT(device_ids_iter->data), recipients_iter->data);
             res = session_cipher_create(&cipher, omemo_ctx.store, &address, omemo_ctx.signal);
-            if (res != 0) {
-                log_error("OMEMO: cannot create cipher for %s device id %d", address.name, address.device_id);
+            if (res != SG_SUCCESS ) {
+                log_error("[OMEMO][SEND] cannot create cipher for %s device id %d - code: %d", address.name, address.device_id, res);
                 continue;
             }
 
             res = session_cipher_encrypt(cipher, key_tag, AES128_GCM_KEY_LENGTH + AES128_GCM_TAG_LENGTH, &ciphertext);
             session_cipher_free(cipher);
-            if (res != 0) {
-                log_error("OMEMO: cannot encrypt key for %s device id %d", address.name, address.device_id);
+            if (res != SG_SUCCESS ) {
+                log_info("[OMEMO][SEND] cannot encrypt key for %s device id %d - code: %d", address.name, address.device_id,res);
                 continue;
             }
             signal_buffer* buffer = ciphertext_message_get_serialized(ciphertext);
@@ -769,8 +842,20 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
 
     g_list_free_full(recipients, free);
 
+    // Don't send the message if no key could be encrypted.
+    // (Since none of the recipients would be able to read the message.)
+    if (keys == NULL) {
+        win_println(win, THEME_ERROR, "!", "This message cannot be decrypted for any recipient.\n"
+                "You should trust your recipients' device fingerprint(s) using \"/omemo fingerprint trust FINGERPRINT\".\n"
+                "It could also be that the key bundle of the recipient(s) have not been received. "
+                "In this case, you could try \"omemo end\", \"omemo start\", and send the message again.");
+        goto out;
+    }
+
+    // Encrypt keys for the sender
     if (!muc) {
         GList* sender_device_id = g_hash_table_lookup(omemo_ctx.device_list, jid->barejid);
+
         for (device_ids_iter = sender_device_id; device_ids_iter != NULL; device_ids_iter = device_ids_iter->next) {
             int res;
             ciphertext_message* ciphertext;
@@ -780,17 +865,23 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
                 .name_len = strlen(jid->barejid),
                 .device_id = GPOINTER_TO_INT(device_ids_iter->data)
             };
+            log_debug("[OMEMO][SEND][Sender] Sending to device %d for %s ", address.device_id, address.name);
+            // Don't encrypt for this device (according to
+            // <https://xmpp.org/extensions/xep-0384.html#encrypt>).
+            if (address.device_id == omemo_ctx.device_id) {
+                continue;
+            }
 
             res = session_cipher_create(&cipher, omemo_ctx.store, &address, omemo_ctx.signal);
             if (res != 0) {
-                log_error("OMEMO: cannot create cipher for %s device id %d", address.name, address.device_id);
+                log_info("[OMEMO][SEND][Sender] cannot create cipher for %s device id %d", address.name, address.device_id);
                 continue;
             }
 
             res = session_cipher_encrypt(cipher, key_tag, AES128_GCM_KEY_LENGTH + AES128_GCM_TAG_LENGTH, &ciphertext);
             session_cipher_free(cipher);
             if (res != 0) {
-                log_error("OMEMO: cannot encrypt key for %s device id %d", address.name, address.device_id);
+                log_info("[OMEMO][SEND][Sender] cannot encrypt key for %s device id %d", address.name, address.device_id);
                 continue;
             }
             signal_buffer* buffer = ciphertext_message_get_serialized(ciphertext);
@@ -805,6 +896,7 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
         }
     }
 
+    // Send the message
     if (muc) {
         ProfMucWin* mucwin = (ProfMucWin*)win;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
@@ -836,7 +928,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
     Jid* sender = NULL;
     Jid* from = jid_create(from_jid);
     if (!from) {
-        log_error("Invalid jid %s", from_jid);
+        log_error("[OMEMO][RECV] Invalid jid %s", from_jid);
         goto out;
     }
 
@@ -851,7 +943,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
     }
 
     if (!key) {
-        log_warning("OMEMO: received a message with no corresponding key");
+        log_warning("[OMEMO][RECV] received a message with no corresponding key");
         goto out;
     }
 
@@ -867,7 +959,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
         }
         g_list_free(roster);
         if (!sender) {
-            log_warning("OMEMO: cannot find MUC message sender fulljid");
+            log_warning("[OMEMO][RECV] cannot find MUC message sender fulljid");
             goto out;
         }
     } else {
@@ -884,12 +976,12 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
 
     res = session_cipher_create(&cipher, omemo_ctx.store, &address, omemo_ctx.signal);
     if (res != 0) {
-        log_error("OMEMO: cannot create session cipher");
+        log_error("[OMEMO][RECV] cannot create session cipher");
         goto out;
     }
 
     if (key->prekey) {
-        log_debug("OMEMO: decrypting message with prekey");
+        log_debug("[OMEMO][RECV] decrypting message with prekey");
         pre_key_signal_message* message;
         ec_public_key* their_identity_key;
         signal_buffer* identity_buffer = NULL;
@@ -922,16 +1014,17 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
 
         if (res == 0) {
             /* Start a new session */
+            log_debug("[OMEMO][RECV] Res is 0 => omemo_bundle_request");
             omemo_bundle_request(sender->barejid, sid, omemo_start_device_session_handle_bundle, free, strdup(sender->barejid));
         }
     } else {
-        log_debug("OMEMO: decrypting message with existing session");
+        log_debug("[OMEMO][RECV] decrypting message with existing session");
         signal_message* message = NULL;
 
         res = signal_message_deserialize(&message, key->data, key->length, omemo_ctx.signal);
 
         if (res < 0) {
-            log_error("OMEMO: cannot deserialize message");
+            log_error("[OMEMO][RECV] cannot deserialize message");
         } else {
             res = session_cipher_decrypt_signal_message(cipher, message, NULL, &plaintext_key);
             *trusted = true;
@@ -941,12 +1034,12 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
 
     session_cipher_free(cipher);
     if (res != 0) {
-        log_error("OMEMO: cannot decrypt message key");
+        log_error("[OMEMO][RECV] cannot decrypt message key");
         goto out;
     }
 
     if (signal_buffer_len(plaintext_key) != AES128_GCM_KEY_LENGTH + AES128_GCM_TAG_LENGTH) {
-        log_error("OMEMO: invalid key length");
+        log_error("[OMEMO][RECV] invalid key length");
         signal_buffer_free(plaintext_key);
         goto out;
     }
@@ -958,7 +1051,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
                             signal_buffer_data(plaintext_key) + AES128_GCM_KEY_LENGTH);
     signal_buffer_free(plaintext_key);
     if (res != 0) {
-        log_error("OMEMO: cannot decrypt message: %s", gcry_strerror(res));
+        log_error("[OMEMO][RECV] cannot decrypt message: %s", gcry_strerror(res));
         free(plaintext);
         plaintext = NULL;
         goto out;
@@ -1053,6 +1146,7 @@ omemo_is_trusted_identity(const char* const jid, const char* const fingerprint)
     buffer = signal_buffer_append(buffer, fingerprint_raw, fingerprint_len);
 
     gboolean trusted = is_trusted_identity(&address, signal_buffer_data(buffer), signal_buffer_len(buffer), &omemo_ctx.identity_key_store);
+    log_debug("[OMEMO] Device trusted %s (%d): %d", jid, GPOINTER_TO_INT(device_id), trusted);
 
     free(fingerprint_raw);
     signal_buffer_free(buffer);
@@ -1132,7 +1226,7 @@ omemo_trust(const char* const jid, const char* const fingerprint_formatted)
 
     GHashTable* known_identities = g_hash_table_lookup(omemo_ctx.known_devices, jid);
     if (!known_identities) {
-        log_warning("OMEMO: cannot trust unknown device: %s", fingerprint_formatted);
+        log_warning("[OMEMO] cannot trust unknown device: %s", fingerprint_formatted);
         cons_show("Cannot trust unknown device: %s", fingerprint_formatted);
         return;
     }
@@ -1143,7 +1237,7 @@ omemo_trust(const char* const jid, const char* const fingerprint_formatted)
     free(fingerprint);
 
     if (!device_id) {
-        log_warning("OMEMO: cannot trust unknown device: %s", fingerprint_formatted);
+        log_warning("[OMEMO] cannot trust unknown device: %s", fingerprint_formatted);
         cons_show("Cannot trust unknown device: %s", fingerprint_formatted);
         return;
     }
@@ -1200,13 +1294,13 @@ omemo_untrust(const char* const jid, const char* const fingerprint_formatted)
     /* Remove existing session */
     GHashTable* known_identities = g_hash_table_lookup(omemo_ctx.known_devices, jid);
     if (!known_identities) {
-        log_error("OMEMO: cannot find known device while untrusting a fingerprint");
+        log_error("[OMEMO] cannot find known device while untrusting a fingerprint");
         goto out;
     }
 
     uint32_t device_id = GPOINTER_TO_INT(g_hash_table_lookup(known_identities, fingerprint));
     if (!device_id) {
-        log_error("OMEMO: cannot find device id while untrusting a fingerprint");
+        log_error("[OMEMO] cannot find device id while untrusting a fingerprint");
         goto out;
     }
     signal_protocol_address address = {
@@ -1246,17 +1340,17 @@ _omemo_log(int level, const char* message, size_t len, void* user_data)
 {
     switch (level) {
     case SG_LOG_ERROR:
-        log_error("OMEMO: %s", message);
+        log_error("[OMEMO][SIGNAL] %s", message);
         break;
     case SG_LOG_WARNING:
-        log_warning("OMEMO: %s", message);
+        log_warning("[OMEMO][SIGNAL] %s", message);
         break;
     case SG_LOG_NOTICE:
     case SG_LOG_INFO:
-        log_info("OMEMO: %s", message);
+        log_debug("[OMEMO][SIGNAL] %s", message);
         break;
     case SG_LOG_DEBUG:
-        log_debug("OMEMO: %s", message);
+        log_debug("[OMEMO][SIGNAL] %s", message);
         break;
     }
 }
@@ -1264,13 +1358,17 @@ _omemo_log(int level, const char* message, size_t len, void* user_data)
 static gboolean
 _handle_own_device_list(const char* const jid, GList* device_list)
 {
+    // We didn't find the own device id -> publish
     if (!g_list_find(device_list, GINT_TO_POINTER(omemo_ctx.device_id))) {
+        cons_show("Could not find own OMEMO device ID. Going to publish own device ID: %d", GINT_TO_POINTER(omemo_ctx.device_id));
+        log_debug("[OMEMO] No device ID for our device. Publishing device list");
         device_list = g_list_copy(device_list);
         device_list = g_list_append(device_list, GINT_TO_POINTER(omemo_ctx.device_id));
         g_hash_table_insert(omemo_ctx.device_list, strdup(jid), device_list);
         omemo_devicelist_publish(device_list);
     }
 
+    log_debug("[OMEMO] Request OMEMO Bundles for our devices");
     GList* device_id;
     for (device_id = device_list; device_id != NULL; device_id = device_id->next) {
         omemo_bundle_request(jid, GPOINTER_TO_INT(device_id->data), omemo_start_device_session_handle_bundle, free, strdup(jid));
@@ -1282,6 +1380,7 @@ _handle_own_device_list(const char* const jid, GList* device_list)
 static gboolean
 _handle_device_list_start_session(const char* const jid, GList* device_list)
 {
+    log_debug("[OMEMO] Start session for %s - device_list", jid);
     omemo_start_session(jid);
 
     return FALSE;
@@ -1387,22 +1486,22 @@ static gboolean
 _load_identity(void)
 {
     GError* error = NULL;
-    log_info("Loading OMEMO identity");
+    log_info("[OMEMO] Loading OMEMO identity");
 
     /* Device ID */
     error = NULL;
     omemo_ctx.device_id = g_key_file_get_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, &error);
     if (error != NULL) {
-        log_error("OMEMO: cannot load device id: %s", error->message);
+        log_error("[OMEMO] cannot load device id: %s", error->message);
         return FALSE;
     }
-    log_info("OMEMO: device id: %d", omemo_ctx.device_id);
+    log_debug("[OMEMO] device id: %d", omemo_ctx.device_id);
 
     /* Registration ID */
     error = NULL;
     omemo_ctx.registration_id = g_key_file_get_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_REGISTRATION_ID, &error);
     if (error != NULL) {
-        log_error("OMEMO: cannot load registration id: %s", error->message);
+        log_error("[OMEMO] cannot load registration id: %s", error->message);
         return FALSE;
     }
 
@@ -1410,7 +1509,7 @@ _load_identity(void)
     error = NULL;
     char* identity_key_public_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, &error);
     if (!identity_key_public_b64) {
-        log_error("OMEMO: cannot load identity public key: %s", error->message);
+        log_error("[OMEMO] cannot load identity public key: %s", error->message);
         return FALSE;
     }
 
@@ -1422,7 +1521,7 @@ _load_identity(void)
     error = NULL;
     char* identity_key_private_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, &error);
     if (!identity_key_private_b64) {
-        log_error("OMEMO: cannot load identity private key: %s", error->message);
+        log_error("[OMEMO] cannot load identity private key: %s", error->message);
         return FALSE;
     }
 
@@ -1599,7 +1698,7 @@ _cache_device_identity(const char* const jid, uint32_t device_id, ec_public_key*
     }
 
     char* fingerprint = _omemo_fingerprint(identity, FALSE);
-    log_info("OMEMO: cache identity for %s:%d: %s", jid, device_id, fingerprint);
+    log_debug("[OMEMO] cache identity for %s:%d: %s", jid, device_id, fingerprint);
     g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
 
     char* device_id_str = g_strdup_printf("%d", device_id);
@@ -1763,6 +1862,7 @@ omemo_parse_aesgcm_url(const char* aesgcm_url,
     }
 
     if (strlen(*fragment) != AESGCM_URL_NONCE_LEN + AESGCM_URL_KEY_LEN) {
+        ret = 1;
         goto out;
     }
 
