@@ -34,7 +34,9 @@
  *
  */
 
-#define _GNU_SOURCE 1
+#include "config.h"
+
+#include "ui.h"
 
 #include <string.h>
 #include <assert.h>
@@ -67,6 +69,8 @@ mucwin_new(const char* const barejid)
     }
 #endif
 
+    // Force redraw here to show correct offline users; before this point muc_members returns a wrong list
+    ui_redraw_all_room_rosters();
     return mucwin;
 }
 
@@ -385,26 +389,36 @@ _mucwin_print_mention(ProfWin* window, const char* const message, const char* co
     while (curr) {
         pos = GPOINTER_TO_INT(curr->data);
 
-        char* before_str = g_strndup(message + last_pos, pos - last_pos);
+        char *before_str = g_utf8_substring(message, last_pos, last_pos + pos - last_pos);
+
         if (strncmp(before_str, "/me ", 4) == 0) {
             win_print_them(window, THEME_ROOMMENTION, ch, flags, "");
             win_append_highlight(window, THEME_ROOMMENTION, "*%s ", from);
             win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str + 4);
         } else {
-            win_print_them(window, THEME_ROOMMENTION, ch, flags, from);
+            // print time and nick only once at beginning of the line
+            if (last_pos == 0) {
+                win_print_them(window, THEME_ROOMMENTION, ch, flags, from);
+            }
             win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
         }
         g_free(before_str);
-        char* mynick_str = g_strndup(message + pos, strlen(mynick));
+
+        glong mynick_len = g_utf8_strlen(mynick, -1);
+        char* mynick_str = g_utf8_substring(message, pos, pos + mynick_len);
         win_append_highlight(window, THEME_ROOMMENTION_TERM, "%s", mynick_str);
         g_free(mynick_str);
 
-        last_pos = pos + strlen(mynick);
+        last_pos = pos + mynick_len;
 
         curr = g_slist_next(curr);
     }
-    if (last_pos < strlen(message)) {
-        win_appendln_highlight(window, THEME_ROOMMENTION, "%s", &message[last_pos]);
+
+    glong message_len = g_utf8_strlen(message, -1);
+    if (last_pos < message_len) {
+        char* rest = g_utf8_substring(message, last_pos, last_pos + message_len);
+        win_appendln_highlight(window, THEME_ROOMMENTION, "%s", rest);
+        g_free(rest);
     } else {
         win_appendln_highlight(window, THEME_ROOMMENTION, "");
     }
